@@ -15,8 +15,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import testtools
 import mock
+import testtools
 
 from troveclient import common
 
@@ -31,15 +31,34 @@ class CommonTest(testtools.TestCase):
             self.assertRaises(Exception,
                               common.check_for_exceptions, resp, "body")
 
-    def test_limit_url(self):
+    def test_append_query_strings(self):
         url = "test-url"
-        self.assertEqual(url, common.limit_url(url, limit=None, marker=None))
+        self.assertEqual(url, common.append_query_strings(url))
+
+        self.assertEqual(url, common.append_query_strings(
+            url, limit=None, marker=None))
 
         limit = "test-limit"
         marker = "test-marker"
-        expected = "test-url?marker=test-marker&limit=test-limit"
-        self.assertEqual(expected,
-                         common.limit_url(url, limit=limit, marker=marker))
+        result = common.append_query_strings(url, limit=limit, marker=marker)
+        self.assertTrue(result.startswith(url + '?'))
+        self.assertIn("limit=%s" % limit, result)
+        self.assertIn("marker=%s" % marker, result)
+        self.assertEqual(result.count('&'), 1)
+
+        opts = {}
+        self.assertEqual(url, common.append_query_strings(
+            url, limit=None, marker=None, **opts))
+
+        opts = {'key1': 'value1', 'key2': None}
+        result = common.append_query_strings(url, limit=None, marker=marker,
+                                             **opts)
+        self.assertTrue(result.startswith(url + '?'))
+        self.assertEqual(result.count('&'), 1)
+        self.assertNotIn("limit=%s" % limit, result)
+        self.assertIn("marker=%s" % marker, result)
+        self.assertIn("key1=%s" % opts['key1'], result)
+        self.assertNotIn("key2=%s" % opts['key2'], result)
 
 
 class PaginatedTest(testtools.TestCase):
